@@ -25,6 +25,7 @@ struct st_gif {
   CGIFRaw*           pGIFRaw;                   // (internal) raw GIF stream
   FILE*              pFile;
   cgif_result        curResult;
+  int                iHEAD;                     // (internal) index to current HEAD frame in aFrames queue
 };
 
 // dimension result type
@@ -73,7 +74,10 @@ CGIF* cgif_newgif(CGIF_Config* pConfig) {
   CGIF*          pGIF;
   CGIFRaw*       pGIFRaw; // raw GIF stream
   CGIFRaw_Config rawConfig = {0};
-
+  // width or heigth cannot be zero
+  if(!pConfig->width || !pConfig->height) {
+    return NULL;
+  }
   pFile = NULL;
   // open output file (if necessary)
   if(pConfig->path) {
@@ -93,6 +97,7 @@ CGIF* cgif_newgif(CGIF_Config* pConfig) {
 
   memset(pGIF, 0, sizeof(CGIF));
   pGIF->pFile = pFile;
+  pGIF->iHEAD = 1;
   memcpy(&(pGIF->config), pConfig, sizeof(CGIF_Config));
   // make a deep copy of global color tabele (GCT), if required.
   if((pConfig->attrFlags & CGIF_ATTR_NO_GLOBAL_TABLE) == 0) {
@@ -150,9 +155,8 @@ static int cmpPixel(const CGIF* pGIF, const CGIF_FrameConfig* pCur, const CGIF_F
   return memcmp(pBefCT + iBef * 3, pCurCT + iCur * 3, 3);
 }
 
-/* optimize GIF file size by only redrawing the rectangular area that differs from previous frame */
-static uint8_t* doWidthHeightOptim(CGIF* pGIF, CGIF_FrameConfig* pCur, CGIF_FrameConfig* pBef, DimResult* pResult) {
-  uint8_t* pNewImageData;
+// compare given frames; returns 0 if frames are equal and 1 if they differ. If they differ, pResult returns area of difference
+static int getDiffArea(CGIF* pGIF, CGIF_FrameConfig* pCur, CGIF_FrameConfig* pBef, DimResult *pResult) {
   const uint8_t* pCurImageData;
   const uint8_t* pBefImageData;
   uint16_t       i, x;
@@ -163,7 +167,7 @@ static uint8_t* doWidthHeightOptim(CGIF* pGIF, CGIF_FrameConfig* pCur, CGIF_Fram
 
   pCurImageData = pCur->pImageData;
   pBefImageData = pBef->pImageData;
-  // find top 
+  // find top
   i = 0;
   while(i < height) {
     for(int c = 0; c < width; ++c) {
@@ -176,16 +180,11 @@ static uint8_t* doWidthHeightOptim(CGIF* pGIF, CGIF_FrameConfig* pCur, CGIF_Fram
     ++i;
   }
 FoundTop:
-  if(i == height) { // need dummy pixel (frame is identical with one before)
-    // TBD we might make it possible to merge identical frames in the future
-    newWidth  = 1;
-    newHeight = 1;
-    newLeft   = 0;
-    newTop    = 0;
-    goto Done;
+  if(i == height) {
+    return 0;
   }
   newTop = i;
-  
+
   // find actual height
   i = height - 1;
   while(i > newTop) {
@@ -207,7 +206,7 @@ FoundHeight:
   while(cmpPixel(pGIF, pCur, pBef, pCurImageData[MULU16(i, width) + x], pBefImageData[MULU16(i, width) + x]) == 0) {
     ++i;
     if(i > (newTop + newHeight - 1)) {
-      ++x; //(x==width cannot happen as goto Done is triggered in the only possible case before)
+      ++x; //(x==width cannot happen as return 0 is trigged in the only possible case before)
       i = newTop;
     }
   }
@@ -219,25 +218,126 @@ FoundHeight:
   while(cmpPixel(pGIF, pCur, pBef, pCurImageData[MULU16(i, width) + x], pBefImageData[MULU16(i, width) + x]) == 0) {
     ++i;
     if(i > (newTop + newHeight - 1)) {
-      --x; //(x<newLeft cannot happen as goto Done is triggered in the only possible case before)
+      --x; //(x<newLeft cannot happen as return 0 is trigged in the only possible case before)
       i = newTop;
     }
   }
   newWidth = (x + 1) - newLeft;
 
-Done:
-
-  // create new image data
-  pNewImageData = malloc(MULU16(newWidth, newHeight)); // TBD check return value of malloc
-  for (i = 0; i < newHeight; ++i) {
-    memcpy(pNewImageData + MULU16(i, newWidth), pCurImageData + MULU16((i + newTop), width) + newLeft, newWidth);
-  }
-  
-  // set new width, height, top, left in DimResult struct
   pResult->width  = newWidth;
   pResult->height = newHeight;
   pResult->top    = newTop;
   pResult->left   = newLeft;
+  return 1;
+}
+
+// compare given global palette frames; returns 0 if frames are equal and 1 if they differ. If they differ, pResult returns area of difference
+static int getDiffAreaGlobalPalette(CGIF* pGIF, CGIF_FrameConfig* pCur, CGIF_FrameConfig* pBef, DimResult *pResult) {
+  const uint8_t* pCurImageData;
+  const uint8_t* pBefImageData;
+  uint32_t       offset;
+  uint16_t       i, x;
+  uint16_t       newHeight, newWidth, newLeft, newTop;
+  const uint16_t width  = pGIF->config.width;
+  const uint16_t height = pGIF->config.height;
+
+  pCurImageData = pCur->pImageData;
+  pBefImageData = pBef->pImageData;
+  // find top
+  i = 0;
+  offset = 0;
+  while(i < height) {
+    if (memcmp(pCurImageData + offset, pBefImageData + offset, width)) {
+      break;
+    }
+    ++i;
+    offset += width;
+  }
+
+  if(i == height) {
+    return 0;
+  }
+  newTop = i;
+
+  // find actual height
+  i = height - 1;
+  offset = MULU16(i, width);
+  while(i > newTop) {
+    if (memcmp(pCurImageData + offset, pBefImageData + offset, width)) {
+      break;
+    }
+    --i;
+    offset -= width;
+  }
+  newHeight = (i + 1) - newTop;
+
+  // find left
+  i = newTop;
+  x = 0;
+  offset = MULU16(i, width);
+  while(pCurImageData[offset + x] == pBefImageData[offset + x]) {
+    ++i;
+    offset += width;
+    if(i > (newTop + newHeight - 1)) {
+      ++x; //(x==width cannot happen as return 0 is triggered in the only possible case before)
+      i = newTop;
+      offset = MULU16(i, width);
+    }
+  }
+  newLeft = x;
+
+  // find actual width
+  i = newTop;
+  x = width - 1;
+  offset = MULU16(i, width);
+  while(pCurImageData[offset + x] == pBefImageData[offset + x]) {
+    ++i;
+    offset += width;
+    if(i > (newTop + newHeight - 1)) {
+      --x; //(x<newLeft cannot happen as return 0 is triggered in the only possible case before)
+      i = newTop;
+      offset = MULU16(i, width);
+    }
+  }
+  newWidth = (x + 1) - newLeft;
+
+  pResult->width  = newWidth;
+  pResult->height = newHeight;
+  pResult->top    = newTop;
+  pResult->left   = newLeft;
+  return 1;
+}
+
+/* optimize GIF file size by only redrawing the rectangular area that differs from previous frame */
+static uint8_t* doWidthHeightOptim(CGIF* pGIF, CGIF_FrameConfig* pCur, CGIF_FrameConfig* pBef, DimResult* pResult) {
+  uint16_t i;
+  uint8_t* pNewImageData;
+  const uint16_t width  = pGIF->config.width;
+  const uint8_t* pCurImageData = pCur->pImageData;
+  int diffFrame;
+
+  if ((pBef->attrFlags & CGIF_FRAME_ATTR_USE_LOCAL_TABLE) == 0 && (pCur->attrFlags & CGIF_FRAME_ATTR_USE_LOCAL_TABLE) == 0
+      && (pBef->attrFlags & CGIF_FRAME_ATTR_HAS_SET_TRANS) == 0 && (pCur->attrFlags & CGIF_FRAME_ATTR_HAS_SET_TRANS) == 0) {
+    // Both frames use global palette; use fast comparison.
+    diffFrame = getDiffAreaGlobalPalette(pGIF, pCur, pBef, pResult);
+  } else {
+    diffFrame = getDiffArea(pGIF, pCur, pBef, pResult);
+  }
+
+  if (diffFrame == 0) { // need dummy pixel (frame is identical with one before)
+    // TBD we might make it possible to merge identical frames in the future
+    pResult->width  = 1;
+    pResult->height = 1;
+    pResult->left   = 0;
+    pResult->top    = 0;
+  }
+
+  // create new image data
+  pNewImageData = malloc(MULU16(pResult->width, pResult->height)); // TBD check return value of malloc
+  for (i = 0; i < pResult->height; ++i) {
+    memcpy(pNewImageData + MULU16(i, pResult->width), pCurImageData + MULU16((i + pResult->top), width) + pResult->left, pResult->width);
+  }
+
   return pNewImageData;
 }
 
@@ -261,12 +361,7 @@ static cgif_result flushFrame(CGIF* pGIF, CGIF_Frame* pCur, CGIF_Frame* pBef) {
   hasSetTransp   = (pCur->config.attrFlags & CGIF_FRAME_ATTR_HAS_SET_TRANS) ? 1 : 0;
   disposalMethod = pCur->disposalMethod;
   transIndex     = pCur->transIndex;
-  // sanity check:
-  // at least one valid CT needed (global or local)
-  if(!useLCT && (pGIF->config.attrFlags & CGIF_ATTR_NO_GLOBAL_TABLE)) {
-    return CGIF_ERROR; // invalid config
-  }
-  // deactivate impossible size optimizations 
+  // deactivate impossible size optimizations
   //  => in case alpha channel is used
   // CGIF_FRAME_GEN_USE_TRANSPARENCY and CGIF_FRAME_GEN_USE_DIFF_WINDOW are not possible
   if(isFirstFrame || hasAlpha) {
@@ -389,14 +484,47 @@ int cgif_addframe(CGIF* pGIF, CGIF_FrameConfig* pConfig) {
     pGIF->curResult = CGIF_ERROR;
     return pGIF->curResult;
   }
+  // sanity check:
+  // at least one valid CT needed (global or local)
+  if(!(pConfig->attrFlags & CGIF_FRAME_ATTR_USE_LOCAL_TABLE) && (pGIF->config.attrFlags & CGIF_ATTR_NO_GLOBAL_TABLE)) {
+    pGIF->curResult = CGIF_ERROR;
+    return CGIF_ERROR; // invalid config
+  }
+
+  // if frame matches previous frame, drop it completely and sum the frame delay
+  if(pGIF->aFrames[pGIF->iHEAD] != NULL) {
+    const uint32_t frameDelay = pConfig->delay + pGIF->aFrames[pGIF->iHEAD]->config.delay;
+    if(frameDelay <= 0xFFFF && !(pGIF->config.genFlags & CGIF_GEN_KEEP_IDENT_FRAMES)) {
+      int sameFrame = 1;
+      if ((pConfig->attrFlags & CGIF_FRAME_ATTR_USE_LOCAL_TABLE) == 0 && (pGIF->aFrames[pGIF->iHEAD]->config.attrFlags & CGIF_FRAME_ATTR_USE_LOCAL_TABLE) == 0
+          && (pConfig->attrFlags & CGIF_FRAME_ATTR_HAS_SET_TRANS) == 0 && (pGIF->aFrames[pGIF->iHEAD]->config.attrFlags & CGIF_FRAME_ATTR_HAS_SET_TRANS) == 0) {
+        if (memcmp(pConfig->pImageData, pGIF->aFrames[pGIF->iHEAD]->config.pImageData, MULU16(pGIF->config.width, pGIF->config.height))) {
+          sameFrame = 0;
+        }
+      } else {
+        for(i = 0; i < pGIF->config.width * pGIF->config.height; i++) {
+          if(cmpPixel(pGIF, pConfig, &pGIF->aFrames[pGIF->iHEAD]->config, pConfig->pImageData[i], pGIF->aFrames[pGIF->iHEAD]->config.pImageData[i])) {
+            sameFrame = 0;
+            break;
+          }
+        }
+      }
+
+      if (sameFrame) {
+        pGIF->aFrames[pGIF->iHEAD]->config.delay = frameDelay;
+        return CGIF_OK;
+      }
+    }
+  }
+
   // search for free slot in frame queue
-  for(i = 1; i < SIZE_FRAME_QUEUE && pGIF->aFrames[i] != NULL; ++i);
+  for(i = pGIF->iHEAD; i < SIZE_FRAME_QUEUE && pGIF->aFrames[i] != NULL; ++i);
   // check whether the queue is full
   // when queue is full: we need to flush one frame.
   if(i == SIZE_FRAME_QUEUE) {
     r = flushFrame(pGIF, pGIF->aFrames[1], pGIF->aFrames[0]);
     freeFrame(pGIF->aFrames[0]);
-    pGIF->aFrames[0] = NULL;
+    pGIF->aFrames[0] = NULL; // avoid potential double free in cgif_close
     // check for errors
     if(r != CGIF_OK) {
       pGIF->curResult = r;
@@ -420,6 +548,7 @@ int cgif_addframe(CGIF* pGIF, CGIF_FrameConfig* pConfig) {
   pNewFrame->disposalMethod        = DISPOSAL_METHOD_LEAVE;
   pNewFrame->transIndex            = 0;
   pGIF->aFrames[i]                 = pNewFrame; // add frame to queue
+  pGIF->iHEAD                      = i;         // update HEAD index
   // check whether we need to adapt the disposal method of the frame before.
   if(pGIF->config.attrFlags & CGIF_ATTR_HAS_TRANSPARENCY) {
     pGIF->aFrames[i]->disposalMethod = DISPOSAL_METHOD_BACKGROUND; // TBD might be removed
@@ -480,7 +609,7 @@ CGIF_CLOSE_Cleanup:
       pGIF->curResult = CGIF_ECLOSE; // error: fclose failed
     }
   }
-  for(int i = 0; i < 3; ++i) {
+  for(int i = 0; i < SIZE_FRAME_QUEUE; ++i) {
     freeFrame(pGIF->aFrames[i]);
   }
 
