@@ -1,6 +1,7 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
+#include <stdio.h>
 
 #include "cgif.h"
 
@@ -8,7 +9,12 @@
 #define HEIGHT 100
 
 static int pWriteFn(void* pContext, const uint8_t* pData, const size_t numBytes) {
-  return fwrite(pData, 1, numBytes, (FILE*) pContext);
+  size_t r = fwrite(pData, 1, numBytes, (FILE*) pContext);
+  if(r == numBytes) {
+    return 0;
+  } else {
+    return -1;
+  }
 }
 
 int main(void) {
@@ -20,6 +26,7 @@ int main(void) {
     0x00, 0x00, 0x00, // black
     0xFF, 0xFF, 0xFF, // white
   };
+  cgif_result r;
 
   FILE* file = fopen("write_fn.gif", "wb");
 
@@ -34,16 +41,26 @@ int main(void) {
   //
   // create new GIF
   pGIF = cgif_newgif(&gConfig);
+  if(pGIF == NULL) {
+    fputs("failed to create new GIF via cgif_newgif()\n", stderr);
+    return 1;
+  }
   //
   // add frames to GIF
   pImageData = malloc(WIDTH * HEIGHT);
   memset(pImageData, 0, WIDTH * HEIGHT);
   fConfig.pImageData = pImageData;
-  cgif_addframe(pGIF, &fConfig);
+  r = cgif_addframe(pGIF, &fConfig);
   free(pImageData);
   //
   // write GIF to file
-  cgif_close(pGIF);                  // free allocated space at the end of the session
+  r = cgif_close(pGIF);                  // free allocated space at the end of the session
   fclose(file);
+
+  // check for errors
+  if(r != CGIF_OK) {
+    fprintf(stderr, "failed to create GIF. error code: %d\n", r);
+    return 2;
+  }
   return 0;
 }
